from __future__ import annotations

from enum import Enum


class Status(str, Enum):
    """Resulting status of some action."""

    success = "success"
    failure = "failure"
    error = "error"
    skipped = "skipped"

    def __str__(self) -> str:
        return self.value

    def __add__(self, other: str) -> str:
        """Error > Failure > Success > Skipped."""
        if self == Status.error or other == Status.error:
            return Status.error

        if self == Status.failure or other == Status.failure:
            return Status.failure

        if self == Status.skipped:
            return other

        return Status.success if other == Status.success else other + self



from enum import Enum


class ParamType(str, Enum):
    """Resulting status of some action."""

    body = "body"
    query = "query"
    params = "params"
    cookies = "cookies"
    headers = "headers"

    def __str__(self) -> str:
        return self.value



from logging import Handler
from typing import Dict, Optional, Tuple

import attr

from .models import Module
from .providers import Provider

CONFIG_VERSION = (1, 0)


@attr.s(slots=True)
class AuthConfig:
    auth_type: str = attr.ib(kw_only=True, default="None")
    username: Optional[str] = attr.ib(kw_only=True, default=None)
    password: Optional[str] = attr.ib(kw_only=True, default=None)
    api_key: Optional[str] = attr.ib(kw_only=True, default=None)
    token: Optional[str] = attr.ib(kw_only=True, default=None)


@attr.s(slots=True, kw_only=True)
class PlanConfig:
    """Test plan configuration."""

    target_url: str = attr.ib(kw_only=True)
    spec_path: Optional[str] = attr.ib(kw_only=True, default=None)
    test_plan_path: Optional[str] = attr.ib(kw_only=True, default=None)
    # This is deprecated and should be removed in next version.
    auth: Optional[Tuple[str, str]] = attr.ib(kw_only=True, default=None)
    auth_type: Optional[str] = attr.ib(kw_only=True, default=None)
    report_to_saas: bool = attr.ib(kw_only=True, default=True)
    auth_config: AuthConfig = attr.ib(kw_only=True)
    headers: Dict[str, str] = attr.ib(kw_only=True, factory=dict)
    env_file_path: Optional[str] = attr.ib(kw_only=True, default=None)
    module_providers: Dict[Module, Provider] = attr.ib(kw_only=True, factory=dict)
    # Current config version
    version = CONFIG_VERSION

    test_case_log_handler: Optional[Handler] = attr.ib(kw_only=True, default=None)
    runner_log_handler: Optional[Handler] = attr.ib(kw_only=True, default=None)

    # Shortcut to convert PlanConfig to dictionary
    asdict = attr.asdict



import attr
import pytest

from levo_commons.events import Finished, Payload


@attr.s(slots=True)
class ExamplePayload(Payload):
    foo: int = attr.ib()


def test_events_api():
    event = Finished(running_time=1.0, payload=ExamplePayload(foo=42))
    assert event.running_time == pytest.approx(1.0)
    assert event.asdict() == {
        "payload": {"foo": 42},
        "running_time": pytest.approx(1.0),
    }
